@@ -1771,6 +1771,7 @@ tidy_mol ()
       for (t = 0; t < d; t++)	/* check against all preceding bonds */
 	{
 	if (tmp_bond->bond == 11) continue;
+	if (hp_bond->decoration || tmp_bond->decoration == 1) continue;
 /* if startpoint or endpoint very close to another, snap to that */
 	  if (abs (hp_bond->x - tmp_bond->x) < 5 &&
 	      abs (hp_bond->y - tmp_bond->y) < 5)
@@ -1934,6 +1935,23 @@ add_r_bracket ()
   Display_Mol ();
 }
 
+void
+add_r2_bracket ()
+{
+
+  int gauge;
+  if (importflag != 0)
+    return;
+  gauge = 10 + (boxgap * mark.h / 600 * mark.w / 600);
+  add_spline (mark.x+gauge,mark.y,  mark.x-gauge,mark.y+gauge/2,
+              mark.x-gauge,mark.y+mark.h-gauge/2, mark.x+gauge,mark.y+mark.h,0,0,curpen);
+  add_spline (mark.x+mark.w-gauge,mark.y,  mark.x+mark.w+gauge,mark.y+gauge/2,
+              mark.x+mark.w+gauge,mark.y+mark.h-gauge/2, mark.x+mark.w-gauge,mark.y+mark.h,0,0,curpen);
+  FreePix ();
+  CreatePix ();
+  Display_Mol ();
+}
+
 void
 add_brace ()
 {
