@@ -17,11 +17,13 @@
 #  endif
 #endif
 
-#define VERSION "1.6.9"
+#define VERSION "1.6.10"
 #define PIXMAPWIDTH 1600
 #define PIXMAPHEIGHT 1200
 
 #define MAXCL 101
+#define BONDTYPES 22
+
 #define Hex 1    /* define angletypes and textdirect. , DONT CHANGE THEM !*/
 #define Pent 2
 #define Pent_switch 3
@@ -220,6 +222,7 @@ extern int export_latex_pic(char *filename, float scalefactor);
 extern int export_svg(char *filename);
 extern int export_latex_pic(char *filename, float scalefactor);
 extern int export_ps_pic(char *filename, float scalefactor);
+extern int export_png_pic(char *filename, float scalefactor);
 extern int print_ps_pic(char *filename);
 extern int export_fw(char *filename);
 extern int export_mdl_mol(FILE *fp, int topipe);
@@ -245,7 +248,6 @@ extern int undo_set_memory_limit(size_t memmax);
 GdkPixmap *picture,*savedpicture,*preview;
 GtkWidget *textbox;
 int xcent,ycent;
-int bondtypes=22;
 int *tmpx,*tmpy;
 char formula[51],weight[21],eweight[21],compos[65];
   
@@ -284,6 +286,7 @@ void Export(void);
 
 extern void add_bracket(void);
 extern void add_r_bracket(void);
+extern void add_r2_bracket(void);
 extern void add_brace(void);
 
 void add_spline(int, int, int, int, int, int, int, int, int, int, int);
@@ -327,7 +330,7 @@ int refx=0,refy=0,refmark=0;
 double bondlen_mm=10.668;
 int db_dist=4;
 int mb_dist=10;
-int epsoption, use_whiteout;
+int epsoption, use_whiteout,use_intlchars;
 int bgred,bggreen,bgblue;
 char bghexcolor[10];
 int curfontsize=3;
