@@ -377,6 +377,8 @@ Add_double (int event_x, int event_y)
    17 - curved half arrow (shortcut)
    18 - uniform triple
    19 - quadruple
+   20 - orbital
+   21 - filled orbital
  */
 {
   struct data *hdc;
@@ -392,6 +394,7 @@ Add_double (int event_x, int event_y)
       else
 	++hdc->bond;
       if (hdc->bond==17) hdc->bond++; /* no bond can become a curved arrow */	
+      if (hdc->bond>19) hdc->bond = 0; /* nor can it turn into an orbital */
       modify = True;
     }
   Display_Mol ();
@@ -406,7 +409,7 @@ Set_bondtype (int event_x, int event_y, int curbond)
   struct xy_co *coord;
   int d;
 
-  if (curbond == 17 && curbond > 19)
+  if (curbond == 17 || curbond > 19)
     return;			/* cannot change anything to 3-point spline */
 
   hdc = select_vector (event_x, event_y, size_factor);
