@@ -152,14 +152,34 @@ Display_Mol ()
 	     hp->n);
   for (d = 0; d < hp->n; d++)
     {
+    int x=hpc->x;
+    int y=hpc->y;
+    int tx=hpc->tx;
+    int ty=hpc->ty;
+#if 1
+	if ( !use_whiteout) {
+		i=has_label(hpc->x,hpc->y);
+		if (i>=0) {
+		int ox=x;
+				x += (4.*size_factor*(i+1)/calc_vector(abs(tx-x),abs(ty-y))) *(tx-x);
+				y += (4.*size_factor*(i+1)/calc_vector(abs(tx-ox),abs(ty-y))) *(ty-y);
+				}
+		i=has_label(hpc->tx,hpc->ty);
+		if (i>=0) {
+		int otx=tx;
+				tx -= (4.*size_factor*(i+1)/calc_vector(abs(tx-x),abs(ty-y))) *(tx-x);
+				ty -= (4.*size_factor*(i+1)/calc_vector(abs(otx-x),abs(ty-y))) *(ty-y);
+				}
+		}		
+#endif
       switch (hpc->bond)
 	{
 	case 0:
-	  Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty, hpc->smarked + hpc->tmarked,hpc->color);	/*single */
+	  Drawline (x, y, tx, ty, hpc->smarked + hpc->tmarked,hpc->color);	/*single */
 	  break;
 	case 1:
 	  {			/*leftdouble */
-	    coord = multi_bonds (hpc->x, hpc->y, hpc->tx, hpc->ty, mb_dist);
+	    coord = multi_bonds (x, y, tx, ty, mb_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
@@ -168,52 +188,52 @@ Display_Mol ()
 	  break;
 	case 2:
 	  {			/*rightdouble */
-	    coord = multi_bonds (hpc->tx, hpc->ty, hpc->x, hpc->y, mb_dist);
+	    coord = multi_bonds (tx, ty, x, y, mb_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
-	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	    Drawline (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  }
 	  break;
 	case 5:
-	  DrawWedge (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawWedge (x, y, tx, ty,
 		     hpc->smarked + hpc->tmarked, hpc->color);
 	  break;
 	case 6:
-	  DrawDashedWedge (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawDashedWedge (x, y, tx, ty,
 			   hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 7:
-	  DrawWiggly (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawWiggly (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 8:
-	  DrawArrow (hpc->x, hpc->y, hpc->tx, hpc->ty, 1,
+	  DrawArrow (x, y, tx, ty, 1,
 		     hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 9:
-	  DrawArrow (hpc->x, hpc->y, hpc->tx, hpc->ty, 2,
+	  DrawArrow (x, y, tx, ty, 2,
 		     hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 10:
-	  DrawWide (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawWide (x, y, tx, ty,
 		    hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 11:
-	  DrawCircle (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawCircle (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 12:
-	  DrawDotted (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawDotted (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 13:
-	  DrawAcross (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawAcross (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 4:
 	  {			/*middouble */
-	    coord = center_double_bond (hpc->x, hpc->y, hpc->tx, hpc->ty, db_dist);
+	    coord = center_double_bond (x, y, tx, ty, db_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	    coord++;
@@ -223,54 +243,54 @@ Display_Mol ()
 	  break;
 	case 3:
 	  {			/*triple */
-	    coord = multi_bonds (hpc->x, hpc->y, hpc->tx, hpc->ty, mb_dist);
+	    coord = multi_bonds (x, y, tx, ty, mb_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
-	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	    Drawline (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	    coord = multi_bonds (hpc->tx, hpc->ty, hpc->x, hpc->y, mb_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
-	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	    Drawline (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  }
 	  break;
 	case 14:
 	  {			/*left partial double */
-	    coord = multi_bonds (hpc->x, hpc->y, hpc->tx, hpc->ty, mb_dist);
+	    coord = multi_bonds (x, y, tx, ty, mb_dist);
 	    DrawDashed (coord->x, coord->y, coord->tx, coord->ty,
 			hpc->smarked + hpc->tmarked,hpc->color);
-	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	    Drawline (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  }
 	  break;
 	case 15:
 	  {			/*right partial double */
-	    coord = multi_bonds (hpc->tx, hpc->ty, hpc->x, hpc->y, mb_dist);
+	    coord = multi_bonds (tx, ty, x, y, mb_dist);
 	    DrawDashed (coord->x, coord->y, coord->tx, coord->ty,
 			hpc->smarked + hpc->tmarked,hpc->color);
-	    Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	    Drawline (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  }
 	  break;
 	case 16:		/*zebra dashed single bond */
-	  DrawStripe (hpc->x, hpc->y, hpc->tx, hpc->ty,
+	  DrawStripe (x, y, tx, ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	  break;
 	case 18:
 	  {			/*triple, equal lengths */
-	    coord = center_double_bond (hpc->x, hpc->y, hpc->tx, hpc->ty, 2*db_dist);
+	    coord = center_double_bond (x, y, tx, ty, 2*db_dist);
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
 	    coord++;
 	    Drawline (coord->x, coord->y, coord->tx, coord->ty,
 		      hpc->smarked + hpc->tmarked,hpc->color);
-	  Drawline (hpc->x, hpc->y, hpc->tx, hpc->ty, hpc->smarked + hpc->tmarked,hpc->color);	/*single */
+	  Drawline (x, y, tx, ty, hpc->smarked + hpc->tmarked,hpc->color);	/*single */
 	  }
 	  break;
 	case 19:
 	  {			/*quadruple */
-	    coord = center_double_bond (hpc->x, hpc->y, hpc->tx, hpc->ty, db_dist+3);
+	    coord = center_double_bond (x, y, tx, ty, db_dist+3);
 	    tmpx1=coord->x;
 	    tmpy1=coord->y;
             tmptx1=coord->tx;
@@ -565,7 +585,7 @@ DrawWide (int x, int y, int tx, int ty, int active,int color)
     thegc = drawing_area->style->bg_gc[GTK_STATE_SELECTED];
   else
   thegc=mygc[color];
-  coord = center_double_bond (x, y, tx, ty, 6);
+  coord = center_double_bond (x, y, tx, ty, 6); 
   mypoints[0].x = (gint16) (coord->x * size_factor);
   mypoints[0].y = (gint16) (coord->y * size_factor);
   mypoints[1].x = (gint16) (coord->tx * size_factor);
@@ -595,7 +615,7 @@ DrawStripe (int x, int y, int tx, int ty, int active, int color)
     thegc = drawing_area->style->bg_gc[GTK_STATE_SELECTED];
   else
   thegc=mygc[color];
-  coord = center_double_bond (x, y, tx, ty, 4);
+  coord = center_double_bond (x, y, tx, ty, 4); 
   mypoints[0].x = (gint16) (coord->x * size_factor);
   mypoints[0].y = (gint16) (coord->y * size_factor);
   mypoints[1].x = (gint16) (coord->tx * size_factor);
@@ -787,6 +807,9 @@ if (len<8) len=8;
  step = 1./len;
  factor=0.0125;
  if (len>20) factor= 0.00125;
+ if (len==8) {
+   factor =0.03;
+ }
   for (i = 1; i < len; i++)
     gdk_draw_line (picture, thegc,
 		   (x + step * i * xlen - factor * ylen * i) * size_factor,
@@ -816,13 +839,16 @@ DrawWedge (int x, int y, int tx, int ty, int active, int color)
     thegc = mygc[color];
   xlen = tx - x;
   ylen = ty - y;
+  
+  float c=1.;
+  if (xlen*xlen+ylen*ylen <=1000.) c=2.;
 
   mypoints[0].x = (gint16) (x * size_factor);
   mypoints[0].y = (gint16) (y * size_factor);
-  mypoints[1].x = (gint16) ((tx - 0.1 * ylen) * size_factor);
-  mypoints[1].y = (gint16) ((ty + 0.1 * xlen) * size_factor);
-  mypoints[2].x = (gint16) ((tx + 0.1 * ylen) * size_factor);
-  mypoints[2].y = (gint16) ((ty - 0.1 * xlen) * size_factor);
+  mypoints[1].x = (gint16) ((tx - 0.1 *c * ylen) * size_factor);
+  mypoints[1].y = (gint16) ((ty + 0.1 *c* xlen) * size_factor);
+  mypoints[2].x = (gint16) ((tx + 0.1 *c* ylen) * size_factor);
+  mypoints[2].y = (gint16) ((ty - 0.1 *c* xlen) * size_factor);
 
   hpc = da_root.next;
   for (d = 0; d < hp->n; d++)
@@ -930,8 +956,8 @@ Drawspline (int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
     beneath the arrowhead */
     xbase= mypoints[20].x;
     ybase= mypoints[20].y;
-    mypoints[20].x = mypoints[20].x - 0.03*(mypoints[20].x-mypoints[19].x);	
-    mypoints[20].y = mypoints[20].y - 0.03*(mypoints[20].y-mypoints[19].y);	
+    mypoints[20].x = mypoints[20].x - (gint16) ( 0.03*(mypoints[20].x-mypoints[19].x));	
+    mypoints[20].y = mypoints[20].y - (gint16) (0.03*(mypoints[20].y-mypoints[19].y));	
     gdk_draw_lines (picture, thegc, mypoints, npoints);
     mypoints[20].x=xbase;
     mypoints[20].y=ybase;
@@ -945,16 +971,16 @@ Drawspline (int x0, int y0, int x1, int y1, int x2, int y2, int x3, int y3,
   if (type <= 0)
     return;
 
-  for (i=0;i<4;i++){ // find acceptable arrowhead length
+  for (i=0;i<4;i++){ /* find acceptable arrowhead length */
   xlen = mypoints[20].x - mypoints[19-i].x;
   ylen = mypoints[20].y - mypoints[19-i].y;
   xbase = mypoints[19-i].x;
   ybase = mypoints[19-i].y;
-  if (xlen*xlen+ylen*ylen >200.0 *size_factor*size_factor) break;
+  if (xlen*xlen+ylen*ylen >200 *size_factor*size_factor) break;
   
   }
 
-  if (xlen*xlen+ylen*ylen >400.0 *size_factor*size_factor) {
+  if (xlen*xlen+ylen*ylen >400 *size_factor*size_factor) {
   xlen -= (mypoints[20].x - mypoints[19].x)/2;
   ylen -= (mypoints[20].y - mypoints[19].y)/2;
   xbase = mypoints[19].x +(mypoints[20].x - mypoints[19].x)/2 ;
@@ -1025,7 +1051,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
   gchar *c;
 #ifdef GTK2
   int dummy;
-  gchar *l="  ";
+  gchar l[8];
   gunichar hc, hl;
   gunichar text[MAXCL];
   gunichar symbol[MAXCL];
@@ -1062,8 +1088,13 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
   y = y * size_factor;
   
 #ifdef GTK2
-  //  c=g_locale_to_utf8(cc,-1,NULL,NULL,NULL);
-  c=g_convert(cc,-1,"UTF-8","ISO8859-1",NULL,NULL,NULL);
+memset(l,0,8*sizeof(gchar));
+    c=g_locale_to_utf8(cc,-1,NULL,NULL,NULL);
+  if (!c)  c=g_convert(cc,-1,"UTF-8","ISO8859-1",NULL,NULL,NULL); 
+  if (!c) {
+  fprintf(stderr,"invalid character in chemtool file\n");
+  return;
+  }
   if (!thelayout) thelayout = gtk_widget_create_pango_layout(drawing_area,"X");
   pango_layout_set_font_description(thelayout,font[fontsize]);
   pango_layout_get_pixel_size(thelayout, &tw, &a);  
@@ -1076,6 +1107,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
   c=cc;
   a = gdk_char_height (font[fontsize], 'X');
   ha = gdk_char_height (font[fontsize], 'x') / 2;
+  a += 6;
   n = (int)strlen (c);
   tw = gdk_char_width (font[fontsize], 'x') ;
 #endif
@@ -1094,8 +1126,9 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 	  if (hc == '}')
 	    {
 	      shiftall = 0;
-	      if (!(hc=g_utf8_get_char(c)))continue;
+	      if (!(c=g_utf8_next_char(c)))continue;
 	      --n;
+	      hc=g_utf8_get_char(c);
 	    }
 	  variance[d] = shiftall;
 	}
@@ -1194,25 +1227,25 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 		}
 	    }
 	}
-      text[d] = *hc++;
+      text[d] = (unsigned char) *hc++;
       if (text[d] == '@' && *hc)
 	{
 	  symbol[d] = TRUE;
-	  text[d] = *hc++;
+	  text[d] = (unsigned char) *hc++;
 	  --n;
 	}
 
       else if (text[d] == '|' && *hc)
 	{
 	  slanted[d] = TRUE;
-	  text[d] = *hc++;
+	  text[d] = (unsigned char) *hc++;
 	  --n;
 	}
 
       else if (text[d] == '#' && *hc)
 	{
 	  bold[d] = TRUE;
-	  text[d] = *hc++;
+	  text[d] = (unsigned char) *hc++;
 	  --n;
 	}
 
@@ -1232,7 +1265,9 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
       for (d = 0; d < nch; d++)
 	{
 #ifdef GTK2
-          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL);
+/*          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL); */
+memset(l,0,8*sizeof(gchar));
+g_unichar_to_utf8(text[d],l);
 #endif          
 	  if (variance[d])
 	    {
@@ -1241,7 +1276,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
               pango_layout_set_font_description(thelayout,smallfont[fontsize]);
               pango_layout_get_pixel_size(thelayout, &tw, &dummy);
 #else             
-	      tw = gdk_char_width (smallfont[fontsize], text[d]) + 2;
+	      tw = gdk_char_width (smallfont[fontsize], (gchar)text[d]) + 2;
 #endif
 	      if (symbol[d])
 #ifdef GTK2
@@ -1251,7 +1286,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-		tw = gdk_char_width (ssymbfont[fontsize], text[d]);
+		tw = gdk_char_width (ssymbfont[fontsize], (gchar)text[d]);
 #endif
 	      if (variance[d] < 0)
 		{
@@ -1273,7 +1308,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_set_font_description(thelayout,font[fontsize]);
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
 #else
-	      tw = 1 + gdk_char_width (font[fontsize], text[d]);
+	      tw = 1 + gdk_char_width (font[fontsize], (gchar)text[d]);
 #endif
 	      if (symbol[d])
 #ifdef GTK2
@@ -1283,7 +1318,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-		tw = gdk_char_width (symbfont[fontsize], text[d]);
+		tw = gdk_char_width (symbfont[fontsize], (gchar)text[d]);
 #endif
 	      if (bold[d])
 #ifdef GTK2
@@ -1293,7 +1328,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	      	tw = gdk_char_width (boldfont[fontsize], text[d]);
+	      	tw = gdk_char_width (boldfont[fontsize], (gchar)text[d]);
 #endif
 	      if (slanted[d])
 #ifdef GTK2
@@ -1303,7 +1338,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-		tw = 1 + gdk_char_width (slfont[fontsize], text[d]);
+		tw = 1 + gdk_char_width (slfont[fontsize], (gchar)text[d]);
 #endif
 	      if (save_x < x) 
 		{
@@ -1318,10 +1353,10 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 	  if (chl == 0)
 	    chl = tw;
 	}
-	if (x != sub_lastx || x != sup_lastx) save_x+= tw; // if the label ends with a sub/superscripted character,
-							   // add its length to the current position (which is before
-							   // the sub/superscript to allow vertical alignment of simultaneous
-							   // sub- and superscripts
+	if (x != sub_lastx || x != sup_lastx) save_x+= tw; /* if the label ends with a sub/superscripted character, */
+							   /* add its length to the current position (which is before */
+							   /* the sub/superscript to allow vertical alignment of simultaneous */
+							   /* sub- and superscripts */
       textl = save_x;
       switch (direct)
 	{
@@ -1337,14 +1372,14 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 	}
     }
   nch = n;
-
   x = hx;
   save_x = sub_lastx = sup_lastx = x;
   for (d = 0; d < n; d++)
     {
 #ifdef GTK2
-//          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL);
-	g_unichar_to_utf8((gunichar)text[d], l);
+/*          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL);*/
+memset(l,0,8*sizeof(gchar));
+	g_unichar_to_utf8(text[d], l);
 #endif          
       if (variance[d])
 	{
@@ -1355,7 +1390,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	  tw = gdk_char_width (smallfont[fontsize], text[d]) + 2;
+	  tw = gdk_char_width (smallfont[fontsize], (gchar)text[d]) + 2;
 #endif
 	  if (symbol[d])
 #ifdef GTK2
@@ -1365,7 +1400,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	    tw = gdk_char_width (ssymbfont[fontsize], text[d]);
+	    tw = gdk_char_width (ssymbfont[fontsize], (gchar)text[d]);
 #endif
 	  if (variance[d] < 0)
 	    {
@@ -1387,7 +1422,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_set_font_description(thelayout,font[fontsize]);
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
 #else                
-	  tw = 1 + gdk_char_width (font[fontsize], text[d]);
+	  tw = 1 + gdk_char_width (font[fontsize], (gchar)text[d]);
 #endif
 	  if (symbol[d])
 #ifdef GTK2
@@ -1397,7 +1432,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	    tw = gdk_char_width (symbfont[fontsize], text[d]);
+	    tw = gdk_char_width (symbfont[fontsize], (gchar)text[d]);
 #endif
 	  if (bold[d])
 #ifdef GTK2
@@ -1407,7 +1442,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	    tw = gdk_char_width (boldfont[fontsize], text[d]);
+	    tw = gdk_char_width (boldfont[fontsize], (gchar)text[d]);
 #endif	    
 	  if (slanted[d])
 #ifdef GTK2
@@ -1417,7 +1452,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	    tw = 1 + gdk_char_width (slfont[fontsize], text[d]);
+	    tw = 1 + gdk_char_width (slfont[fontsize], (gchar)text[d]);
 #endif
 	  if (save_x < x)
 	    {
@@ -1438,7 +1473,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 		    y - a + ha - 5 * size_factor + variance[d],
 		    tw + 8 * size_factor, a + 10 * size_factor, 0, 360 * 64);
 	else
-      gdk_draw_arc (picture, background_gc, 1,
+      if (use_whiteout) gdk_draw_arc (picture, background_gc, 1,
 		    x - 2 * size_factor,
 		    y -a + ha + size_factor * variance[d],
 		    tw + 8 * size_factor, a + 2 * size_factor, 0, 360 * 64);
@@ -1453,16 +1488,15 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
   y -= a ;
 #endif
   
+ 
   save_x = sub_lastx = sup_lastx = x;
   for (d = 0; d < n; d++)
     {
 #ifdef GTK2
-	memset(l,0,sizeof(l));
-//#ifdef GTK2
-//          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL);
-//	memset(l,0,2);
-	g_unichar_to_utf8((gunichar)text[d], l);
-
+	memset(l,0,8*sizeof(gchar));
+/*#ifdef GTK2*/
+/*          l = g_locale_to_utf8((const gchar*) &text[d],1,NULL,NULL,NULL);*/
+g_unichar_to_utf8(text[d],l);
 	if (symbol[d]) {
 	unsigned int unicodechar=text[d]+848;
 	
@@ -1555,7 +1589,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
 	g_unichar_to_utf8((gunichar)unicodechar,l);
 	}
 #else      
-      hl[0] = text[d];
+      hl[0] = (char)text[d];
       hl[1] = 0;
       l = &hl[0];
 #endif
@@ -1569,7 +1603,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	  tw = gdk_char_width (smallfont[fontsize], text[d]) + 2;
+	  tw = gdk_char_width (smallfont[fontsize], (gchar)text[d]) + 2;
 #endif
 	  if (symbol[d])
 #ifdef GTK2
@@ -1579,7 +1613,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
                   pango_layout_get_pixel_size(thelayout, &tw, &dummy);
                 }
 #else                
-	    tw = gdk_char_width (ssymbfont[fontsize], text[d]);
+	    tw = gdk_char_width (ssymbfont[fontsize], (gchar)text[d]);
 #endif
 
 	  if (variance[d] < 0)
@@ -1634,7 +1668,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
               pango_layout_set_font_description(thelayout,font[fontsize]);
               pango_layout_get_pixel_size(thelayout,&tw,&dummy);
 #else
-	  tw = 1 + gdk_char_width (font[fontsize], text[d]);
+	  tw = 1 + gdk_char_width (font[fontsize], (gchar)text[d]);
 #endif	  
 	  if (symbol[d])
 #ifdef GTK2
@@ -1644,7 +1678,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
               pango_layout_get_pixel_size(thelayout,&tw,&dummy);
              } 
 #else
-	    tw = gdk_char_width (symbfont[fontsize], text[d]);
+	    tw = gdk_char_width (symbfont[fontsize], (gchar)text[d]);
 #endif	    
 	  if (bold[d])
 #ifdef GTK2
@@ -1654,7 +1688,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
               pango_layout_get_pixel_size(thelayout,&tw,&dummy);
              } 
 #else
-	    tw = gdk_char_width (boldfont[fontsize], text[d]);
+	    tw = gdk_char_width (boldfont[fontsize], (gchar)text[d]);
 #endif	    
 	  if (slanted[d])
 #ifdef GTK2
@@ -1664,7 +1698,7 @@ Drawstring (int x, int y, char *cc, int direct, int active, int thecolor, int se
               pango_layout_get_pixel_size(thelayout,&tw,&dummy);
              } 
 #else
-	    tw = 1 + gdk_char_width (slfont[fontsize], text[d]);
+	    tw = 1 + gdk_char_width (slfont[fontsize],(gchar) text[d]);
 #endif
 	  if (save_x < x)
 	    {
