@@ -605,8 +605,8 @@ undo_memory_stream (UNDO_MEMORY * memory)
     UNDO_ERROR_NULL (UNDO_NOMEM);
 
   stream->implementation = memory;
-  stream->destroy = (void *) undo_stream_destroy;
-  stream->read = (void *) undo_stream_read;
+  stream->destroy = /*(void * )*/  undo_stream_destroy;
+  stream->read = /*(void *)*/ undo_stream_read;
 
   return (UNDO_MEMORY_STREAM *) stream;
 }
